package g55803.dev2.td1;
import static g55803.dev2.td1.ArrayUtil.shuffle;
import static g55803.dev2.td1.Lecture.readNbrPairs;
import static g55803.dev2.td1.Lecture.readUserInt;

/**
 * @author Nathan Furnal
 */

public class Memory {
    /** Créé un tableau de paires identiques
     * non mélangées.
     *
     * @param n nombre de paires à créer.
     * @return Un tableau de n paires.
     */
    public static int[] initCards(int n){
        if (n < 3 || n > 20){
            throw new IllegalArgumentException("Demandez entre 3 et 20 paires !");
        }
        int[] tab = new int[2*n];
        int j = 1;
        for (int i = 0; i < 2*n; i += 2) {
            tab[i] = j;
            tab[i+1] = j;
            j++;
        }
        return tab;
    }

    /** Demande une position de cartes parmi les cartes diponibles
     * à l'utilisateur. Si la carte est déjà retournée ou bien que la sélection
     * est en dehors des cartes jouables, la position est demandée à nouveau.
     * @param cards Tableau des cartes à jouer.
     * @param collectedCards Tableau de booléen relevant les cartes retournées ou pas.
     * @return Vrai si le jeu est terminé et faux autrement.
     */
    public static int askPosition(int[] cards, boolean[] collectedCards){
        int minPos = 0;
        int maxPos = cards.length - 1;
        String message = "Entrez une position  de carte (" + minPos + " à " + maxPos + ") :";
        int pos = readUserInt(message);
        while (pos < minPos || pos  > maxPos || collectedCards[pos]){
            System.out.println("Cette carte n'est pas disponible !");
            pos = readUserInt(message);
        }
        return pos;
    }

    /** Vérifie si les cartes sélectionnées sont les mêmes ou pas et lance un message
     * d'erreur dans le cas où la même carte est sélectionnée deux fois.
     * @param cards Tableau des cartes à jouer.
     * @param collectedCards Tableau de booléen relevant les cartes retournées ou pas.
     * @param pos1 Position de la première carte retournée.
     * @param pos2 Position de la seconde carte retournée.
     */
    public static void checkPosition(int[] cards, boolean[] collectedCards, int pos1, int pos2){
        if (pos1 == pos2){
            throw new IllegalArgumentException("Vous ne pouvez pas sélectionner la même carte deux fois!");
        }
        if (cards[pos1] == cards[pos2]){
            System.out.println("Elles correspondent !");
            collectedCards[pos1] = collectedCards[pos2] = true;
        }
        else {
            System.out.println("Elles ne correspondent pas !");
        }

    }

    /** Vérifie si toutes les cartes sont retournées et donc si le jeu est fini.
     * @param collectedCards Tableau de booléen relevant les cartes retournées ou pas.
     * @return Si le jeu est terminé ou pas.
     */
    public static boolean isGameOver(boolean[] collectedCards){
        // cool check https://stackoverflow.com/a/8260897/9479128
        for (boolean b : collectedCards)
            if (!b) return false;
        return true;
    }

    public static int playMemory(int n){
        int turns = 1;
        int[] cards = initCards(n);
        boolean[] collectedCards = new boolean[2*n];
        shuffle(cards);
        while (!isGameOver(collectedCards)){
            MemoryUtil.displayCards(cards, collectedCards);
            System.out.println("*** tour " + turns);
            int pos1 = askPosition(cards, collectedCards);
            System.out.println("La carte en position " + pos1 + " est : " + cards[pos1]);
            int pos2 = askPosition(cards, collectedCards);
            System.out.println("La carte en position " + pos2 + " est : " + cards[pos2]);
            checkPosition(cards, collectedCards, pos1, pos2);
            turns++;
        }
        return turns;
    }

    public static void main(String[] args) {
        System.out.println("*** Memory ***");
        int nbrPairs = readNbrPairs();
        int turns = playMemory(nbrPairs);
        System.out.println("Partie terminée en " + turns + " coups.");
    }
}
