package g55803.dev2.td2;

/** Provides a Moment class to represent a moment of the day
 * with hours, minutes and seconds.
 * @author Nathan Furnal
 */
public class Moment {
    private int second;
    private int minute;
    private int hour;

    /** Class constructor.
     * @param hr The number of hours.
     * @param min The number of minutes.
     * @param sec The number of seconds.
     */
    public Moment(int hr, int min, int sec) {
        if (hr < 0 || hr > 23){
            throw new IllegalArgumentException("Hours values should lie in the [0,23] range : " + hr);
        }
        if (min < 0 || min > 59){
            throw new IllegalArgumentException("Minutes values should lie in the [0, 59] range: " + min);
        }

        if (sec < 0 || sec > 59){
            throw new IllegalArgumentException("Seconds values should lie in hte [0, 59] range : " + sec);
        }
        this.second = sec;
        this.minute = min;
        this.hour = hr;
    }

    /** Provides a shorthand to create moments
     * with no parameters, it defaults to zero values
     * for all parameters.
     *
     */
    public Moment(){
        this(0, 0, 0);
    }

    /** Gets the seconds.
     * @return Number of seconds in the moment.
     */
    public int getSecond(){
        return this.second;
    }

    /** Gets the minutes.
     * @return Number of minutes in the moment.
     */
    public int getMinute(){
        return this.minute;
    }

    /** Gets the hours.
     * @return Number of hours in the moment.
     */
    public int getHour(){
        return this.hour;
    }

    /** Returns the string representation of the Moment object.
     * @return The representation in the following form : 'hours:minutes:seconds'.
     */
    @Override
    public String toString() {
        return ""+getHour()+":"+getMinute()+":"+getSecond();
    }

    /** Provides an equality check with other objects. If the other objects
     * are null or aren't of type Moment then it will return false. It will otherwise
     * check for similar hours, minutes and seconds.
     * @param obj The object to be compared to.
     * @return An equality check with the current instance of the Moment class.
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null){
            return false;
        }
        if (!(obj instanceof Moment)){
            return false;
        }
        if (obj == this){
            return true;
        }
        Moment o = (Moment) obj;
       return (o.getSecond() == this.getSecond() &&
               o.getMinute() == this.getMinute() &&
               o.getHour() == this.getHour());
    }

    /** Provides a hash code for the Moment instances.
     * Some values are hardcoded, ideally it should provide a simple
     * way to check if two moments are not equal. Meaning that different moment should
     * not have the same hash code.
     * @return A hash code.
     */
    @Override
    public int hashCode() {
        int hash = 17;
        hash = hash * getSecond() + hash;
        hash = hash * getMinute() + hash;
        hash = hash * getHour() + hash;
        return (hash + 11)*4;
    }

    /** Converts the moment to its value in seconds.
     * @return The converted values, in seconds.
     */
    public int toSeconds(){
        int hrToSec = this.getHour()*3600;
        int minToSec = this.getMinute()*60;
        return hrToSec + minToSec + this.getSecond();
    }

    /** Compares moments and checks whether they represent the same amount of time.
     * @param other The moment to be compared to.
     * @return -1 if the first moment comes before the other, 1 if it comes after and 0 if they're equal.
     * If something goes terribly wrong it returns -999.
     */
    public int compareTo(Moment other){
        int res = -999;
        int otherSeconds = other.toSeconds();
        int thisSeconds = this.toSeconds();
        if (thisSeconds == otherSeconds) {
            res = 0;
        }
        if (thisSeconds < otherSeconds){
            res = -1;
        }

        if (thisSeconds > otherSeconds){
            res = 1;
        }
        return res;
    }
}
