package g55803.dev2.td3;

import java.util.Objects;

/** Implémentation d'une carte à jouer pour le jeu Memory.
 * @author Nathan Furnal
 */
public class Card {
    private int value;
    private boolean hidden;

    /** Constructeur de la carte à jouer avec sa valeur et si elle est cachée ou pas.
     * @param val valeur de la carte, entier strictement positif.
     * @param hidden si la carte est cachée ou pas.
     */
    public Card(int val, boolean hidden){
        if(val <= 0){
            throw new IllegalArgumentException("La valeur doit doit être positive : " + val);
        }
        this.value = val;
        this.hidden = hidden;
    }

    /** Constructeur qui implémente une carte cachée par défaut.
     * @param val valeur de la carte.
     */
    public Card(int val){
        this(val, true);
    }

    /** Révèle une carte face cachée.
     *
     */
    public void reveal(){
        if(this.hidden){
            this.hidden = false;
        }
    }

    /** Représentation textuelle de la carte à jouer, qui présente sa valeur
     * si elle est révélée et un point d'interrogation sinon.
     * @return La valeur de la carte ou ?.
     */
    @Override
    public String toString() {
        if (this.hidden){
            return "?";
        }
        else {
            return "" + this.value;
        }
    }

    /** Etablit une comparaison entre deux cartes, basée sur la valeur.
     * @param o Un autre objet auquel comparer la carte.
     * @return égalité basée sur la valeur.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Card card = (Card) o;
        return value == card.value;
    }

    /** Crée le hash code lié à une carte.
     * @return Un hash code.
     */
    @Override
    public int hashCode() {
        int hash = 73;
        hash = hash * this.value + 11;
        return hash;
    }

    /** Getter pour savoir si la carte est cachée ou pas.
     * @return Vrai si la carte est cachée et faux autrement.
     */
    public boolean isHidden(){
        return this.hidden;
    }

    /** Getter pour obtenir la valeur de la carte.
     * @return La valeur de la carte.
     */
    public int getValue(){
        return this.value;
    }

}
