package g55803.dev2.td3;
import java.util.Arrays;
import java.util.Random;

/** Implémentation du jeu Memory, composé de cartes.
 * @author Nathan Furnal
 */
public class Game {
    private Card[] cards;

    /** Crée un jeu de de cartes composé de 3 à 20 paires.
     * @param n Nombre de paires de cartes.
     * @throws IllegalArgumentException si les paires ne sont pas comprises entre 3 et 20.
     */
    public Game(int n){
        if (n < 3 || n > 20) {
            throw new IllegalArgumentException("Demandez entre 3 et 20 paires !");
        }
        cards = new Card[2*n];
        int j = 1;
        for (int i = 0; i < 2 * n; i+=2) {
            cards[i] = new Card(j);
            cards[i+1] = new Card(j);
            j++;
        }
    }

    /** Mélange les cartes.
     *
     */
    public void shuffle() {
        Card[] array = this.cards;
        int index;
        Random random = new Random();
        for (int i = array.length - 1; i > 0; i--) {
            index = random.nextInt(i + 1);
            swap(array, i, index);
        }
    }

    /**
     * Échange le contenu de deux cases de tableau. Échange dans le tableau
     * <code>array</code> le contenu des cases en positions <code>pos1</code> et
     * <code>pos2</code>. Ces 2 positions sont supposées correctes.
     *
     * @param array le tableau dont on échange 2 cases.
     * @param pos1 la position de la première case à échanger.
     * @param pos2 la position de la seconde case à échanger.
     * @throws IllegalArgumentException si au moins une des positions est
     * strictement négative.
     */
    public void swap(Card[] array, int pos1, int pos2) {
        if (pos1 < 0 || pos2 < 0) {
            throw new IllegalArgumentException("Les positions doivent être des "
                    + "entiers positifs " + pos1 + " " + pos2);
        }
        Card savedValue = array[pos1];
        array[pos1] = array[pos2];
        array[pos2] = savedValue;
    }

    /** Donne le nombre de cartes dans le jeu.
     * @return Longueur du jeu.
     */
    public int getSize(){
        return cards.length;
    }

    /** Donne la qualité cachée d'une carte précise dans le jeu.
     * @param pos La position d'une carte dans le jeu.
     * @return Si la carte donnée est cachée ou pas.
     */
    public boolean isCardHidden(int pos){
        return cards[pos].isHidden();
    }

    /** Donne la valeur d'une carte précise dans le jeu.
     * @param pos La position d'une carte dans le jeu.
     * @return La valeur de la carte à la position donnée.
     */
    public int getCardValue(int pos){
        return cards[pos].getValue();
    }

    /** Vérifie si deux cartes sont égales ou pas dans le jeu et les retourne si c'est le cas.
     * @param pos1 La position de la première carte.
     * @param pos2 La position de la seconde carte.
     * @return La vérification de si les cartes sont égales en valeur.
     */
    public boolean checkPositions(int pos1, int pos2){
        if(pos1 == pos2){
            throw new IllegalArgumentException("Les positions doivent êtres différentes :" + pos1 + "," + pos2);
        }
        if (cards[pos1].equals(cards[pos2])){
            cards[pos1].reveal();
            cards[pos2].reveal();
            return true;
        }
        return false;
    }

    /** Vérifie si le jeu est terminé ou pas. On le considère terminé quand toutes les cartes sont retournées.
     * @return Si le jeu est terminé.
     */
    public boolean isGameOver(){
        for(Card c : cards){
            if(c.isHidden()){
                return false;
            }
        }
        return true;
    }

    /** Représentation textuelle du jeu de cartes avec les cartes cachées et révélées.
     * @return La représentation du jeu.
     */
    @Override
    public String toString() {
        StringBuilder str = new StringBuilder();
        for (Card c : cards){
            str.append(c.toString()).append(" ");
        }
        return str.toString();
    }

    /** Compare deux jeux de cartes.
     * @param o Un autre objet à comparer.
     * @return Si deux jeux ont des cartes de même valeur, dans le même ordre.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Game game = (Game) o;

        if(this.getSize() != game.getSize()){
            return false;
        }

        for (int i = 0; i < this.getSize(); i++) {
            if (this.getCardValue(i) != game.getCardValue(i)){
                return false;
            }
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 31;
        for (Card c : cards){
            hash += c.getValue() * 3;
        }
        return hash;
    }
}
