package g55803.dev2.td5;

/**
 * @author Nathan Furnal
 */
public class MagicSquare {
    private final int n;
    private final int[][] values;

    /**
     * Constructeur du carré magique.
     *
     * @param order Ordre du carré magique.
     */
    public MagicSquare(int order) {
        if (order < 3 || order % 2 == 0) {
            throw new IllegalArgumentException("L'ordre doit être impair et supérieur ou égal à 3 : " + order
            );
        }
        this.n = order;
        values = new int[n][n];
        int row = n - 1;
        int col = n / 2;
        int num = 1;
        int newRow = 0, newCol = 0;
        while (num <= n * n) {
            if (row < 0 || row >= n)
                row = 0;
            if (col < 0 || col >= n)
                col = 0;
            if (values[row][col] == 0) {
                newRow = row;
                newCol = col;
                values[row++][col++] = num++;
            } else {
                row = newRow - 1;
                col = newCol;
            }
        }
    }

    /**
     * Constructeur par défaut.
     */
    public MagicSquare() {
        n = 3;
        values = new int[][]{{4, 9, 2}, {3, 5, 7}, {8, 1, 6}};
    }

    /**
     * Répresentation textuelle du carré magique.
     *
     * @return Une chaîne de caractère avec les valeurs du carré.
     */
    @Override
    public String toString() {
        int len = String.valueOf(n * n).length() + 1;
        StringBuilder s = new StringBuilder();
        for (int[] arr : values) {
            s.append("\n");
            for (int el : arr) {
                String val = String.valueOf(el);
                int spaces = len - val.length();
                s.append(" ").append(el).append(" ".repeat(spaces));
            }
        }
        return s.toString();
    }

    /**
     * Donne l'ordre du carré.
     *
     * @return L'ordre.
     */
    public int getOrder() {
        return n;
    }

    /**
     * Donne les valeurs qui composent le carré.
     *
     * @return Un tableau en deux dimensions contenant les valeurs.
     */
    public int[][] getValues() {
        return values;
    }

    /**
     * Calcule la somme d'une ligne du carré.
     *
     * @param num Le numéro de la ligne.
     * @return La somme de la ligne sélectionnée.
     */
    public int sumRow(int num) {
        if (num < 0 || num >= n) {
            throw new IllegalArgumentException("La ligne demandée doit être dans le carré : " + num);
        }
        int sum = 0;
        for (int el : values[num])
            sum += el;
        return sum;
    }

    /**
     * Calcule la somme d'une colonne du carré.
     *
     * @param num Le numéro de la colonne.
     * @return La somme de la colonne sélectionnée.
     */
    public int sumColumn(int num) {
        if (num < 0 || num >= n) {
            throw new IllegalArgumentException("La colonne demandée doit être dans le carré : " + num);
        }
        int sum = 0;
        for (int[] arr : values)
            sum += arr[num];
        return sum;
    }

    /**
     * Calcule la somme d'une des deux diagonales.
     *
     * @param num 1 pour la première diagonale et 2 pour la seconde.
     * @return La somme d'une des deux diagonales.
     */
    public int sumDiag(int num) {
        int sum = 0;
        if (!(num == 1 || num == 2)) {
            throw new IllegalArgumentException("Il faut sélectionner la première (1) ou seconde (2) diagonale : " + num);
        }
        if (num == 1) {
            for (int i = 0; i < n; i++) {
                sum += values[i][i];
            }
        }
        if (num == 2) {
            for (int i = 0; i < n; i++) {
                sum += values[i][n - 1 - i];
            }
        }
        return sum;
    }

    public boolean isMagicSquare() {
        int base = sumRow(0);
        int sumDiag1 = sumDiag(1);
        int sumDiag2 = sumDiag(2);
        if (base != sumDiag1 || base != sumDiag2)
            return false;
        int i = 0;
        while (i < n) {
            if (base != sumRow(i) || base != sumColumn(i)) {
                return false;
            } else i++;
        }
        return i == n;
    }

    public void displayMagicSquare() {
        System.out.println();
        System.out.println("Square order : " + getOrder());
        int len = String.valueOf(n * n).length() + 1;
        String fmt = "%" + len + "d";
        for (int i = 0; i < n; i++) {
            System.out.println();
            for (int j = 0; j < n; j++) {
                System.out.printf(fmt, values[i][j]);
                System.out.print(" ".repeat(len));
            }
            System.out.print("-> " + sumRow(i));
        }
        System.out.println();
        String fmt2 = "%" + len + "s";
        for (int i = 0; i < n; i++) {
            System.out.printf(fmt2, "↓");
            System.out.print(" ".repeat(len));
        }
        System.out.println();
        for (int i = 0; i < n; i++) {
            System.out.printf(fmt, sumColumn(i));
            System.out.print(" ".repeat(len));
        }
    }
}
