package g55803.dev2.td6;

import java.util.Iterator;

/**
 * @author Nathan Furnal
 */
public class Range implements Iterable<Integer>, Comparable<Range>{
    private final int lowerBound;
    private final int upperBound;

    public Range(int lower, int upper){
        this.lowerBound = lower;
        this.upperBound = upper;
    }

    public int getLowerBound() {
        return lowerBound;
    }

    public int getUpperBound() {
        return upperBound;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Range range = (Range) o;

        if (getLowerBound() != range.getLowerBound()) return false;
        return getUpperBound() == range.getUpperBound();
    }

    @Override
    public int hashCode() {
        int result = getLowerBound();
        result = 31 * result + getUpperBound();
        return result;
    }

    @Override
    public String toString() {
        return "Range(" + lowerBound + ", " + upperBound + ")";
    }

    @Override
    public Iterator<Integer> iterator() {
        return new RangeIterator(getLowerBound(), getUpperBound());
    }

    @Override
    public int compareTo(Range r) {
        if(getLowerBound() < r.getLowerBound()){
            return -1;
        }
        else if (getLowerBound() == r.getLowerBound()){
            return getUpperBound() < r.getUpperBound() ? -1 : 0;
        }
        else {return 1;}
    }

    class RangeIterator implements Iterator<Integer>{
        private int current;
        private final int upperBound;
        public RangeIterator(int lowerBound, int upperBound){
            this.current = lowerBound;
            this.upperBound = upperBound;
        }

        @Override
        public boolean hasNext() {
            return current < upperBound;
        }

        @Override
        public Integer next() {
            return current++;
        }
    }
}
